package it.fbk.sra.ejade.builder;

import it.fbk.sra.ejade.EjadePlugin;
import it.fbk.sra.ejade.preferences.PreferenceConstants;

import java.io.File;
import java.util.ArrayList;

import org.eclipse.core.resources.ICommand;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IProjectNature;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jdt.core.ClasspathContainerInitializer;
import org.eclipse.jdt.core.IClasspathContainer;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;

public class JADENature implements IProjectNature {

	/**
	 * ID of this project nature
	 */
	public static final String NATURE_ID = "it.fbk.sra.ejade.jadeNature";

	private IProject project;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.resources.IProjectNature#configure()
	 */
	public void configure() throws CoreException {
		// Add JADE/JADEX library to the project
		String ejadeHome = EjadePlugin.getDefault().getHome();
		String[] libPaths;
		if (EjadePlugin.getDefault().getPlatform().equals(
				PreferenceConstants.PLATFORM_JADE)) {
			libPaths = new String[] { ejadeHome + "lib" + File.separatorChar
					+ "libjade" };
		} else {
			libPaths = new String[] {
					ejadeHome + "lib" + File.separatorChar + "libjade",
					ejadeHome + "lib" + File.separatorChar + "libjadex", };
		}

		try {
			createLibraryWithAllJar(project, EjadePlugin.getDefault().getPlatform(), libPaths);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			// e.printStackTrace();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.resources.IProjectNature#deconfigure()
	 */
	public void deconfigure() throws CoreException {
		// Remove project library from the project
		String userLibName = EjadePlugin.getDefault().getPlatform();
		IJavaProject jproject = JavaCore.create(project);

		final IPath userlibPath = new Path(JavaCore.USER_LIBRARY_CONTAINER_ID)
				.append(userLibName);

		ArrayList<IClasspathEntry> newEntries = new ArrayList<IClasspathEntry>();
		
		IClasspathEntry entry[] = jproject.getRawClasspath();
		for (IClasspathEntry en : entry) {
			if (en.getEntryKind() == IClasspathEntry.CPE_CONTAINER) {
				if (!en.getPath().equals(userlibPath))
					newEntries.add(en);
			} else 
				newEntries.add(en);
		}
		
		jproject.setRawClasspath(newEntries
				.toArray(new IClasspathEntry[newEntries.size()]), null);
		jproject.save(null, false);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.resources.IProjectNature#getProject()
	 */
	public IProject getProject() {
		return project;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.resources.IProjectNature#setProject(org.eclipse.core.resources.IProject)
	 */
	public void setProject(IProject project) {
		this.project = project;
	}

	/**
	 * Creates a library in the project with the list of all jar contained in
	 * the folder jarFolder.
	 * 
	 * FIXME After Eclipse has been restarted, the container is unbound
	 * 
	 * @param project
	 *            the eclipse project
	 * @param userlibName
	 *            the linrary name
	 * @param jarFolder
	 *            the folder that contain jars
	 */
	public void createLibraryWithAllJar(IProject project,
			final String userlibName, final String libPaths[]) throws Exception {
		Assert.isNotNull(project);
		Assert.isNotNull(userlibName);
		Assert.isTrue(userlibName.length() > 0);

		final IJavaProject jproject = JavaCore.create(project);
		final IPath userlibPath = new Path(JavaCore.USER_LIBRARY_CONTAINER_ID)
				.append(userlibName);

		IClasspathEntry entry[] = jproject.getRawClasspath();
		for (IClasspathEntry en : entry) {
			if (en.getEntryKind() == IClasspathEntry.CPE_CONTAINER) {
				if (en.getPath().equals(userlibPath))
					return;
			}
		}

		// String[] ss = JavaCore.getUserLibraryNames();
		// for (String s: ss){
		// System.out.println(s);
		// }
		
		// If not exist the library, create new one
		
		ClasspathContainerInitializer initializer = JavaCore
				.getClasspathContainerInitializer(JavaCore.USER_LIBRARY_CONTAINER_ID);
		IJavaProject fDummyProject = createPlaceholderProject();

		ArrayList<IClasspathEntry> jarEntries = new ArrayList<IClasspathEntry>();
		for (String libPath : libPaths) {
			IPath pathElement = new Path(libPath);
			File file = pathElement.toFile();
			File[] members = file.listFiles();
			if (members == null)
				break;
			for (File f : members) {
				if (f.getName().endsWith(".jar")) {
					jarEntries.add(JavaCore.newLibraryEntry(new Path(f
							.getAbsolutePath()), null, null, false));
				}
			}
		}
		
		JADEClasspathContainer updatedContainter = new JADEClasspathContainer(
				jarEntries.toArray(new IClasspathEntry[jarEntries.size()]), userlibName, userlibPath);
		
		try {
			initializer.requestClasspathContainerUpdate(userlibPath, fDummyProject,
					updatedContainter);
		} catch (CoreException e) {
		}
		
		
		// Update to current project
		ArrayList<IClasspathEntry> newEntries = new ArrayList<IClasspathEntry>();
		for (IClasspathEntry en : entry) {
			newEntries.add(en);
		}

		IClasspathEntry varEntry = JavaCore.newContainerEntry(userlibPath,
				false); // not exported

		newEntries.add(varEntry);
		
		jproject.setRawClasspath(newEntries
				.toArray(new IClasspathEntry[newEntries.size()]), null);
		jproject.save(null, true);

	}

	private class JADEClasspathContainer implements IClasspathContainer {
		private IClasspathEntry[] entries;
		private String libName;
		private IPath libPath;

		public JADEClasspathContainer(IClasspathEntry[] entries,
				String libName, IPath libPath) {
			super();
			this.entries = entries;
			this.libName = libName;
			this.libPath = libPath;
		}

		public IClasspathEntry[] getClasspathEntries() {
			return entries;
		}

		public String getDescription() {
			return libName;
		}

		public int getKind() {
			return IClasspathContainer.K_APPLICATION;
		}

		public IPath getPath() {
			return libPath;
		}

	}

	private static IJavaProject createPlaceholderProject() {
		String name = "####internal"; //$NON-NLS-1$
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		while (true) {
			IProject project = root.getProject(name);
			if (!project.exists()) {
				return JavaCore.create(project);
			}
			name += '1';
		}
	}
}
