package it.fbk.sra.ejade.platform;

import it.fbk.sra.ejade.EjadePlugin;
import it.fbk.sra.ejade.util.CommonUtil;
import it.fbk.sra.ejade.util.LauchingUtil;

import java.io.File;
import java.util.List;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.debug.ui.DebugUITools;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.internal.launching.JavaSourceLookupDirector;
import org.eclipse.jdt.launching.IJavaLaunchConfigurationConstants;
import org.eclipse.jdt.launching.IRuntimeClasspathEntry;
import org.eclipse.jdt.launching.JavaRuntime;
import org.eclipse.jdt.launching.sourcelookup.containers.JavaProjectSourceContainer;
import org.eclipse.jface.dialogs.MessageDialog;

public class JADEPlatform implements IPlatform,
		IJavaLaunchConfigurationConstants {

	/**
	 * Get the platform launching params
	 * @return
	 */
	public void debug(String agentParams, IJavaProject project, String configurationName) {
		String ejadeHome = EjadePlugin.getDefault().getHome();
		try {
			// Create new launching configuration
			ILaunchConfigurationWorkingCopy workingCopy =  LauchingUtil.prepareWorkingConfig("Debug JADE Agent: " + configurationName);
			if (workingCopy == null)
				return;

			workingCopy.setAttribute(ATTR_MAIN_TYPE_NAME, "jade.Boot");
			workingCopy.setAttribute(ATTR_PROGRAM_ARGUMENTS, " -container" + CommonUtil.launchingParams() + agentParams);
			
			workingCopy.setAttribute(ATTR_VM_ARGUMENTS, "-Dhttp.proxyHost="+ EjadePlugin.getDefault().getProxyHost() 
					+ " -Dhttp.proxyPort=" + EjadePlugin.getDefault().getProxyPort());

			String[] libPaths = new String[] { "lib" + File.separatorChar
					+ "libjade" };
			List<String> classpath = LauchingUtil.prepareClasspath(project,
					libPaths);

			if (classpath == null) {
				MessageDialog.openError(EjadePlugin.getDefault().getWorkbench()
								.getActiveWorkbenchWindow().getShell(),
								"Error",
								"Can not build classpath, please check JADE lib directory in the plugin home");
				return;
			}

			workingCopy.setAttribute(ATTR_CLASSPATH, classpath);
			workingCopy.setAttribute(ATTR_DEFAULT_CLASSPATH, false);
			workingCopy.setAttribute(DebugPlugin.getUniqueIdentifier() + ".appendEnvironmentVariables", true);

			//workingCopy.setAttribute(ATTR_SOURCE_PATH, (List)null);
			//workingCopy.setAttribute(ATTR_DEFAULT_SOURCE_PATH, (String)null);
			JavaSourceLookupDirector sourceLookup = new JavaSourceLookupDirector();
			JavaProjectSourceContainer projectSource = new JavaProjectSourceContainer(project);
			sourceLookup.setSourceContainers(projectSource.getSourceContainers());
			
			workingCopy.setAttribute(ILaunchConfiguration.ATTR_SOURCE_LOCATOR_ID, "org.eclipse.jdt.launching.sourceLocator.JavaSourceLookupDirector");
			workingCopy.setAttribute(ILaunchConfiguration.ATTR_SOURCE_LOCATOR_MEMENTO, sourceLookup.getMemento());
			
			File workingDir;
			if (project != null){
				workingCopy.setAttribute(ATTR_PROJECT_NAME, project.getElementName());
				workingDir = project.getOutputLocation().toFile();
			} else {
				workingDir = (new Path(ejadeHome)).toFile();
			}
			workingCopy.setAttribute(ATTR_WORKING_DIRECTORY, workingDir.getAbsolutePath());
			
			// Save configuration and launch JADE
			ILaunchConfiguration configuration;
			configuration = workingCopy.doSave();
			DebugUITools.launch(configuration, ILaunchManager.DEBUG_MODE);
		} catch (CoreException e) {
			e.printStackTrace();
		}

	}

	public void deploy(String agentParams, IJavaProject project, String configurationName) {
		String ejadeHome = EjadePlugin.getDefault().getHome();
		try {
			// Create new launching configuration
			ILaunchConfigurationWorkingCopy workingCopy =  LauchingUtil.prepareWorkingConfig("Deploy JADE Agent: " + configurationName);
			if (workingCopy == null)
				return;

			workingCopy.setAttribute(ATTR_MAIN_TYPE_NAME, "jade.Boot");
			
			workingCopy.setAttribute(ATTR_PROGRAM_ARGUMENTS, " -container" + CommonUtil.launchingParams() + agentParams);
			
			workingCopy.setAttribute(ATTR_VM_ARGUMENTS, "-Dhttp.proxyHost="+ EjadePlugin.getDefault().getProxyHost() 
					+ " -Dhttp.proxyPort=" + EjadePlugin.getDefault().getProxyPort());

			String[] libPaths = new String[] { "lib" + File.separatorChar
					+ "libjade" };
			List<String> classpath = LauchingUtil.prepareClasspath(project,
					libPaths);

			if (classpath == null) {
				MessageDialog.openError(EjadePlugin.getDefault().getWorkbench()
								.getActiveWorkbenchWindow().getShell(),
								"Error",
								"Can not build classpath, please check JADE lib directory in the plugin home");
				return;
			}

			workingCopy.setAttribute(ATTR_CLASSPATH, classpath);
			workingCopy.setAttribute(ATTR_DEFAULT_CLASSPATH, false);
			workingCopy.setAttribute("org.eclipse.debug.core.appendEnvironmentVariables", true);

			File workingDir;
			if (project != null){
				workingCopy.setAttribute(ATTR_PROJECT_NAME, project.getElementName());
				workingDir = project.getOutputLocation().toFile();
			} else {
				workingDir = (new Path(ejadeHome)).toFile();
			}
			workingCopy.setAttribute(ATTR_WORKING_DIRECTORY, workingDir.getAbsolutePath());
			
			// Save configuration and launch JADE
			ILaunchConfiguration configuration;

			configuration = workingCopy.doSave();
			DebugUITools.launch(configuration, ILaunchManager.RUN_MODE);
		} catch (CoreException e) {
			e.printStackTrace();
		}
	}

	public void shutdown() {
		String ejadeHome = EjadePlugin.getDefault().getHome();
		try {
			// Create new launching configuration
			ILaunchConfigurationWorkingCopy workingCopy =  LauchingUtil.prepareWorkingConfig("Shutdown JADE");
			if (workingCopy == null)
				return;
			
			workingCopy.setAttribute(ATTR_MAIN_TYPE_NAME, "jade.Boot");
			workingCopy.setAttribute(ATTR_PROGRAM_ARGUMENTS, " -container" + CommonUtil.launchingParams() + " KillerAgent:" + KillerAgent.class.getName());

			String[] libPaths = new String[] { "lib" + File.separatorChar
					+ "libjade" };
			List<String> classpath = LauchingUtil.prepareClasspath(null,
					libPaths);

			if (classpath == null) {
				MessageDialog.openError(EjadePlugin.getDefault().getWorkbench()
								.getActiveWorkbenchWindow().getShell(),
								"Error",
								"Can not build classpath, please check JADE lib directory in the plugin home");
				return;
			}
			
			IPath path = CommonUtil.getPluginPath();
			IRuntimeClasspathEntry pathEntry;
			IPath pathElement = path.append("ejade.jar");
			pathEntry = JavaRuntime.newArchiveRuntimeClasspathEntry(pathElement);
			pathEntry.setClasspathProperty(IRuntimeClasspathEntry.USER_CLASSES);
			classpath.add(pathEntry.getMemento());

			workingCopy.setAttribute(ATTR_CLASSPATH, classpath);
			workingCopy.setAttribute(ATTR_DEFAULT_CLASSPATH, false);

			File workingDir = (new Path(ejadeHome)).toFile();
			workingCopy.setAttribute(ATTR_WORKING_DIRECTORY, workingDir
					.getAbsolutePath());

			// Save configuration and launch JADE
			ILaunchConfiguration configuration;

			configuration = workingCopy.doSave();
			DebugUITools.launch(configuration, ILaunchManager.RUN_MODE);
		} catch (CoreException e) {
			e.printStackTrace();
		}
	}

	public boolean start() {

		String ejadeHome = EjadePlugin.getDefault().getHome();
		String mtp_port = EjadePlugin.getDefault().getMtpPort();
		boolean isStartEcat = EjadePlugin.getDefault().isStartEcatRemote();
		
		try {
			// Create new launching configuration
			ILaunchConfigurationWorkingCopy workingCopy =  LauchingUtil.prepareWorkingConfig("Start JADE");
			if (workingCopy == null)
				return false;
			
			workingCopy.setAttribute(ATTR_MAIN_TYPE_NAME, "jade.Boot");
			
			String arguments = " -gui";
			if (EjadePlugin.getDefault().isUseRandomMtp())
				arguments +=  CommonUtil.launchingParams();
			else 
				arguments +=  " -jade_mtp_http_port " + mtp_port +  CommonUtil.launchingParams();
			
			if (isStartEcat)
				arguments += " RemoteMonitorAgent:it.itc.sra.ecat.core.remote.RemoteMonitorAgent";

			workingCopy.setAttribute(ATTR_PROGRAM_ARGUMENTS, arguments);
			
			String[] libPaths;
			if (isStartEcat)
				libPaths = new String[] { "lib" + File.separatorChar
					+ "libjade", "lib" + File.separatorChar + "ecatlib"};
			else 
				libPaths = new String[] { "lib" + File.separatorChar
					+ "libjade"};
			
			List<String> classpath = LauchingUtil.prepareClasspath(null, libPaths);

			if (classpath == null) {
				MessageDialog.openError(EjadePlugin.getDefault().getWorkbench()
								.getActiveWorkbenchWindow().getShell(),
								"Error",
								"Can not build classpath, please check JADE lib directory in the plugin home");
				return false;
			}

			workingCopy.setAttribute(ATTR_CLASSPATH, classpath);
			workingCopy.setAttribute(ATTR_DEFAULT_CLASSPATH, false);

			File workingDir = (new Path(ejadeHome)).toFile();
			workingCopy.setAttribute(ATTR_WORKING_DIRECTORY, workingDir
					.getAbsolutePath());

			// Save configuration and launch JADE
			ILaunchConfiguration configuration;

			configuration = workingCopy.doSave();
			DebugUITools.launch(configuration, ILaunchManager.RUN_MODE);
		} catch (CoreException e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}

}
