package it.fbk.sra.ejade.popup;

import java.util.List;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

public class AgentNameWizardPage extends WizardPage {

	private List<AgentParam> agentParams;
	
	public AgentNameWizardPage(String pageName) {
		super(pageName);

	}

	public AgentNameWizardPage(String pageName, String title,
			ImageDescriptor titleImage) {
		super(pageName, title, titleImage);

	}

	public void createControl(Composite parent) {
		Composite composite = new Composite(parent, SWT.NONE);
		
		//Group myGroup = new Group(composite, SWT.BORDER);
		//myGroup.setText("Agents list");
		GridLayout gridLayout = new GridLayout ();
		gridLayout.numColumns = 4;
		gridLayout.makeColumnsEqualWidth = true;
		gridLayout.marginLeft = 5;
		gridLayout.marginRight = 5;
		composite.setLayout (gridLayout);
		
		
		Label label = new Label(composite, SWT.BOLD);
		label.setText("Agent class");
		GridData data = new GridData ();
		data.grabExcessHorizontalSpace = true;
		label.setLayoutData (data);
		
		label = new Label(composite, SWT.BOLD);
		label.setText("Agent name");
		data = new GridData ();
		data.grabExcessHorizontalSpace = true;
		label.setLayoutData (data);

		label = new Label(composite, SWT.BOLD);
		label.setText("Agent parameters");
		data = new GridData ();
		data.grabExcessHorizontalSpace = true;
		label.setLayoutData (data);

		label = new Label(composite, SWT.BOLD);
		label.setText("Number of clones");
		data = new GridData ();
		data.grabExcessHorizontalSpace = true;
		label.setLayoutData (data);
		
		if (agentParams != null) {
		
			for (AgentParam agent : agentParams){	
			
				String agentClass = agent.getFullClassName();
				String agentName = agent.getName();
				
				label = new Label(composite, SWT.NONE);
				
				/**
				 * Modified May 05 2009, show only the last segment
				 */
				int lastDotPosition = agentClass.lastIndexOf(".");
				if (lastDotPosition < 0) lastDotPosition = 0;
				
				label.setText(agentClass.substring(lastDotPosition));
				
				data = new GridData ();
				data.grabExcessHorizontalSpace = true;
				label.setLayoutData (data);
				
				Text text = new Text(composite, SWT.BORDER);
				text.setText(agentName);
				text.addModifyListener(new MyListener(agentClass, Field.NAME));
				
				data = new GridData ();
				data.horizontalAlignment = GridData.FILL;
				data.grabExcessHorizontalSpace = true;
				text.setLayoutData (data);
				
				text = new Text(composite, SWT.BORDER);
				text.setText(agent.getParam());
				text.addModifyListener(new MyListener(agentClass, Field.PARAMS));
				
				data = new GridData ();
				data.horizontalAlignment = GridData.FILL;
				data.grabExcessHorizontalSpace = true;
				text.setLayoutData (data);
				
				text = new Text(composite, SWT.BORDER);
				text.setText(String.valueOf(agent.getNumClones()));
				text.addModifyListener(new MyListener(agentClass, Field.NUMCLONES));
				
				data = new GridData ();
				data.horizontalAlignment = GridData.FILL;
				data.grabExcessHorizontalSpace = true;
				text.setLayoutData (data);
				
			}
		}
		setControl(composite);
	}

	/**
	 * Field type
	 * @author cunduy
	 *
	 */
	private enum Field{
		NAME("agentName"),
		PARAMS("params"),
		NUMCLONES("clones");
		private final String value;
		
		Field(String v) {
	        value = v;
	    }

	    public String value() {
	        return value;
	    }
	};
	
	/**
	 * Change listener
	 * @author cunduy
	 *
	 */
	private class MyListener implements ModifyListener{

		private String key;
		private Field field;
		
		public MyListener(String key, Field field) {
			this.key = key;
			this.field = field;
		}

		public void modifyText(ModifyEvent e) {

			if (agentParams != null) {
				for (AgentParam agent : agentParams){	
					if (agent.getFullClassName().equals(key)){
						if (field.equals(Field.NAME))
							agent.setName(((Text)e.getSource()).getText());
						else if (field.equals(Field.PARAMS))
							agent.setParam(((Text)e.getSource()).getText());
						else if (field.equals(Field.NUMCLONES)) {
							String tmp = ((Text)e.getSource()).getText();
							int numClones = 1;
							try {
								numClones = Integer.valueOf(tmp);
							} catch (NumberFormatException ex) {
								// skip
							}
							agent.setNumClones(numClones);
						}
						break;
					}
				}
			}
		}
	}

	public List<AgentParam> getAgentParams() {
		return agentParams;
	}

	public void setAgentParams(List<AgentParam> agentParams) {
		this.agentParams = agentParams;
	}
	
}
