package it.fbk.sra.ejade.popup;

import it.fbk.sra.ejade.EjadePlugin;
import it.fbk.sra.ejade.platform.IPlatform;
import it.fbk.sra.ejade.platform.JADEPlatform;
import it.fbk.sra.ejade.platform.JADEXPlatform;
import it.fbk.sra.ejade.popup.AgentParam.AgentType;
import it.fbk.sra.ejade.preferences.PreferenceConstants;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.widgets.Event;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.actions.ActionDelegate;

public class Deploy extends ActionDelegate implements IObjectActionDelegate {

	IWorkbenchPart targetPart;
	IFile[] selectedFiles;

	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
		this.targetPart = targetPart;
	}

	@SuppressWarnings("restriction")
	@Override
	public void run(IAction action) {
		if (selectedFiles != null) {
			List<AgentParam> deployList = new ArrayList<AgentParam>();

			IJavaProject myJavaProject = null;
			for (int i = 0; i < selectedFiles.length; i++) {
				IFile file = selectedFiles[i];
				IProject thisProject = file.getProject();
				try {
					if (thisProject.hasNature("org.eclipse.jdt.core.javanature")) {
						// Get the handle of the project
						myJavaProject = JavaCore.create(thisProject);

						// Get the handle of the java file

						String extension = file.getFileExtension();

						String packageName = "";
						String nameWithoutExtention = "";

						if (extension.equals("java")) {
							IJavaElement element = JavaCore.create(file);
							if (element == null) {
								MessageDialog.openError(targetPart.getSite().getShell(), "Launch error", "File "
										+ file.getName() + " need to be compiled before running !");
							} else {

								ICompilationUnit myJavaClass = (ICompilationUnit) element;
								IJavaElement packageFragment = myJavaClass.getParent();

								packageName = packageFragment.getElementName();
								if (!packageName.equals(""))
									packageName = packageName + ".";

								String tmp = myJavaClass.getElementName();
								nameWithoutExtention = tmp.substring(0, tmp.length() - 5); // trip
																							// .java
							}
						} else if (extension.equals("xml")) {
							String tmp = file.getName();
							if (tmp.length() > 10)
								nameWithoutExtention = tmp.substring(0, tmp.length() - 10); // trip
																							// .agent.xml
							else
								nameWithoutExtention = tmp.substring(0, tmp.length() - 4); // trip
																							// .xml

							IPath path = file.getFullPath();
							int j, k = 1;
							boolean match = false;
							IClasspathEntry[] entry = myJavaProject.getResolvedClasspath(true);

							for (j = 0; j < entry.length; j++) {
								if (entry[j].getEntryKind() == IClasspathEntry.CPE_SOURCE) {
									IPath rootPath = entry[j].getPath();
									// check for matching
									match = false;
									if (rootPath.segmentCount() < path.segmentCount()) {
										k = 0;
										while (k < rootPath.segmentCount())
											if (rootPath.segment(k).equals(path.segment(k)))
												k++;
									}
									if (k == rootPath.segmentCount()){
										match = true;
										break;
									}
								}
							}

							if (!match)
								k = 2; // suppose that project has the following structure [project]/[src]

							for (j = k; j < path.segmentCount() - 1; j++) {
								packageName += path.segment(j) + ".";
							}

						}

						String agentClass = packageName + nameWithoutExtention;
						String agentName = EjadePlugin.getDefault().getPluginPreferences().getString(agentClass);
						String params = EjadePlugin.getDefault().getPluginPreferences().getString(
								agentClass + "_params");

						if (agentName == null || agentName == "") {
							agentName = nameWithoutExtention;
						}

						AgentParam agentParam = new AgentParam(agentName, agentClass);
						if (extension.equals("xml"))
							agentParam.setType(AgentType.JADEX_AGENT);
						else
							agentParam.setType(AgentType.JADE_AGENT);

						agentParam.setParam(params);

						deployList.add(agentParam);
					}
				} catch (CoreException e) {
					e.printStackTrace();
				}
			}

			// Show the dialog
			AgentNameWizard nameWizard = new AgentNameWizard(deployList);
			WizardDialog dialog = new WizardDialog(EjadePlugin.getDefault().getWorkbench().getActiveWorkbenchWindow()
					.getShell(), nameWizard);

			if (dialog.open() == Window.OK) {
				// Build agent list
				String agentList = "";
				String agentName = "";
				for (AgentParam agent : deployList) {
					String agentClass = agent.getFullClassName();
					agentName = agent.getName();
					String params = agent.getParam();
					// Save
					EjadePlugin.getDefault().getPluginPreferences().setValue(agentClass, agentName);
					EjadePlugin.getDefault().getPluginPreferences().setValue(agentClass + "_params", params);
					if (agent.getType().equals(AgentType.JADE_AGENT)) {
						// JADE
						if (params != null && !params.equals("")) {
							for (int i = 0; i < agent.getNumClones(); i++) {
								if (i == 0)
									agentList += agentName + ":" + agentClass + "(" + params + ") ";
								else
									agentList += agentName + "_" + String.valueOf(i) + ":" + agentClass + "(" + params + ") ";
							}
						} else {
							for (int i = 0; i < agent.getNumClones(); i++) {
								if (i == 0)
									agentList += agentName + ":" + agentClass + " ";
								else
									agentList += agentName + "_" + String.valueOf(i) + ":" + agentClass + " ";
							}
						}
					} else {
						// JADEX
						if (params != null && !params.equals("")) {
							for (int i = 0; i < agent.getNumClones(); i++) {
								if (i == 0)
									agentList += agentName + ":jadex.adapter.jade.JadeAgentAdapter(" + agentClass + " \"" + params + "\") ";
								else
									agentList += agentName + "_" + String.valueOf(i) + ":jadex.adapter.jade.JadeAgentAdapter(" + agentClass + " \""		+ params + "\") ";
							}
						} else {
							for (int i = 0; i < agent.getNumClones(); i++) {
								if (i == 0)
									agentList += agentName + ":jadex.adapter.jade.JadeAgentAdapter(" + agentClass + " default) ";
								else 
									agentList += agentName + "_" + String.valueOf(i) + ":jadex.adapter.jade.JadeAgentAdapter(" + agentClass	+ " default) ";
							}
						}
					}
				}

				if (agentList != "") {
					IPlatform platform;
					if (EjadePlugin.getDefault().getPlatform().endsWith(PreferenceConstants.PLATFORM_JADE)) {
						platform = new JADEPlatform();
					} else {
						platform = new JADEXPlatform();
					}
					platform.deploy(agentList, myJavaProject, agentName);
				} else
					MessageDialog.openError(this.targetPart.getSite().getShell(), "Launching Error",
							"Selected file(s) are not in a Java Project");
			}
		}
	}

	@Override
	public void runWithEvent(IAction action, Event event) {
		this.run(action);
	}

	@Override
	public void selectionChanged(IAction action, ISelection selection) {
		if (!EjadePlugin.getDefault().isReady()) {
			action.setEnabled(false);
			selectedFiles = null;
			return;
		}
		ArrayList<IFile> newFiles = new ArrayList<IFile>();
		IFile file = null;
		if (selection != null && selection instanceof IStructuredSelection) {
			IStructuredSelection ss = (IStructuredSelection) selection;
			for (Iterator<?> iter = ss.iterator(); iter.hasNext();) {
				Object obj = iter.next();
				if (obj instanceof IFile) {
					file = (IFile) obj;
				} else if (obj instanceof IAdaptable) {
					IAdaptable a = (IAdaptable) obj;
					IResource res = (IResource) a.getAdapter(IResource.class);
					if (res instanceof IFile) {
						file = (IFile) res;
					}
				}
				if (file != null && file.isSynchronized(IResource.DEPTH_ZERO)) {
					newFiles.add(file);
				}
			}
		}

		if (newFiles.isEmpty()) {
			selectedFiles = null;
			action.setEnabled(false);
		} else {
			selectedFiles = newFiles.toArray(new IFile[newFiles.size()]);
			action.setEnabled(true);
		}
	}

	@Override
	public void init(IAction action) {
		selectedFiles = null;
	}

}
